from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.group_role import GroupRole
from ..models.user_membership_role_type_1 import UserMembershipRoleType1


T = TypeVar("T", bound="UserMembership")


@_attrs_define
class UserMembership:
    """UserMembership model

    Attributes:
        id (str):
        role (Union[GroupRole, UserMembershipRoleType1]):
    """

    id: str
    role: Union[GroupRole, UserMembershipRoleType1]

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        id = self.id
        role: str
        if isinstance(self.role, GroupRole):
            role = self.role.value
        else:
            role = self.role.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "id": id,
                "role": role,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`UserMembership` from a dict"""
        d = src_dict.copy()
        id = d.pop("id")

        def _parse_role(data: object) -> Union[GroupRole, UserMembershipRoleType1]:
            try:
                if not isinstance(data, str):
                    raise TypeError()
                role_type_0 = GroupRole(data)

                return role_type_0
            except:  # noqa: E722
                pass
            if not isinstance(data, str):
                raise TypeError()
            role_type_1 = UserMembershipRoleType1(data)

            return role_type_1

        role = _parse_role(d.pop("role"))

        user_membership = cls(
            id=id,
            role=role,
        )

        return user_membership



"""Displays the built-in/unbranded sign out page."""
from http import HTTPStatus
from typing import Any
from typing import Dict
from typing import Union

import httpx

from ... import errors
from ...client import AuthenticatedClient
from ...client import Client
from ...types import Response


def _get_kwargs() -> Dict[str, Any]:
    _kwargs: Dict[str, Any] = {
        "method": "post",
        "url": "/auth/signout",
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Any:
    if response.status_code == HTTPStatus.NO_CONTENT:
        return None

    raise errors.UnexpectedStatus(response.status_code, response.content)


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
) -> Response[Any]:
    """Displays the built-in/unbranded sign out page.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs()

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
) -> Response[Any]:
    """Displays the built-in/unbranded sign out page.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs()

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define

from ..models.item_type import ItemType


T = TypeVar("T", bound="SearchCollectionsResponseHitsItemItemsItem")


@_attrs_define
class SearchCollectionsResponseHitsItemItemsItem:
    """SearchCollectionsResponseHitsItemItemsItem model

    Attributes:
        id (str):
        name (str):
        path (str):
        type (ItemType):
        version_id (str):
    """

    id: str
    name: str
    path: str
    type: ItemType
    version_id: str

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        id = self.id
        name = self.name
        path = self.path
        type = self.type.value
        version_id = self.version_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "id": id,
                "name": name,
                "path": path,
                "type": type,
                "versionId": version_id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`SearchCollectionsResponseHitsItemItemsItem` from a dict"""
        d = src_dict.copy()
        id = d.pop("id")

        name = d.pop("name")

        path = d.pop("path")

        type = ItemType(d.pop("type"))

        version_id = d.pop("versionId")

        search_collections_response_hits_item_items_item = cls(
            id=id,
            name=name,
            path=path,
            type=type,
            version_id=version_id,
        )

        return search_collections_response_hits_item_items_item



from .auth import AuthCommands
from .base_commands import BaseCommands
from .cde import CdeCommands
from .pdd import PddCommands


class Commands(BaseCommands):
    auth = AuthCommands
    cdeset = CdeCommands
    pdd = PddCommands



from pypigeon.pigeon_core.api.cde import cdes_delete_cdeset
from pypigeon.pigeon_core.api.cde import cdes_get_cdeset
from pypigeon.pigeon_core.api.cde import cdes_list_cdesets
from pypigeon.pigeon_core.api.cde import cdes_new_cdeset
from pypigeon.pigeon_core.api.data_dictionaries import (
    collections_dictionaries_copy_to_cdeset,
)
from pypigeon.pigeon_core.models import CdesNewCdesetBody
from pypigeon.pigeon_core.models import CollectionsDictionariesCopyToCdesetBody

from .base_commands import BaseCommands


class CdeCommands(BaseCommands):
    """Manipulating CDE sets"""

    @BaseCommands._with_arg("cdeset_name", help="name of new CDE set")
    @BaseCommands._with_arg("description", help="short description for the new CDE set")
    def new(self) -> None:
        """Create a new CDE set.

        Admin access is required.

        """
        cdes_new_cdeset.sync(
            body=CdesNewCdesetBody(
                name=self.args.cdeset_name, description=self.args.description
            ),
            client=self.core,
        )

    def list(self) -> None:
        """List available CDE sets."""
        cdeset_list = cdes_list_cdesets.sync(client=self.core)

        self._output(cdeset_list.to_dict())

    @BaseCommands._with_arg("cdeset_name", help="name of CDE set to delete")
    def delete(self) -> None:
        """Delete a CDE set.

        Admin access is required.

        """
        cdes_delete_cdeset.sync(cdeset_name=self.args.cdeset_name, client=self.core)

    @BaseCommands._with_arg("cdeset_name", help="name of CDE set to update")
    @BaseCommands._with_arg("collection_id", help="collection UUID")
    @BaseCommands._with_arg("version_id", help="collection version, typically 'LIVE'")
    @BaseCommands._with_arg("item_id", help="item UUID")
    @BaseCommands._with_arg(
        "--description", help="optional update to CDE set description"
    )
    def copy_from_pdd(self) -> None:
        """Copy data elements into a CDE set from a PDD.

        Admin access is required, as well as access to the specified PDD.

        """
        if self.args.description:
            description = self.args.description
        else:
            cdeset_info = cdes_get_cdeset.sync(
                cdeset_name=self.args.cdeset_name, client=self.core
            )
            description = cdeset_info.description

        collections_dictionaries_copy_to_cdeset.sync(
            collection_id=self.args.collection_id,
            version_id=self.args.version_id,
            item_id=self.args.item_id,
            body=CollectionsDictionariesCopyToCdesetBody(
                name=self.args.cdeset_name,
                description=description,
            ),
            client=self.core,
        )



import datetime
from typing import Any
from typing import cast
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define
from dateutil.parser import isoparse

from ..models.collection_metadata import CollectionMetadata
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="Collection")


@_attrs_define
class Collection:
    """Core collection properties, with IDs

    Attributes:
        account_id (str): The account containing this collection
        created_on (datetime.datetime):
        description (str): The collection's full description
        id (str):
        name (str): The collection's short human-readable name
        version_ids (List[str]): All available versions of the collection
        metadata (Union[Unset, CollectionMetadata]): Arbitrary user-defined metadata, key-value pairs
    """

    account_id: str
    created_on: datetime.datetime
    description: str
    id: str
    name: str
    version_ids: List[str]
    metadata: Union[Unset, "CollectionMetadata"] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        account_id = self.account_id
        created_on = self.created_on.isoformat()
        description = self.description
        id = self.id
        name = self.name
        version_ids = self.version_ids

        metadata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = self.metadata.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "account_id": account_id,
                "createdOn": created_on,
                "description": description,
                "id": id,
                "name": name,
                "versionIds": version_ids,
            }
        )
        if metadata is not UNSET:
            field_dict["metadata"] = metadata

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`Collection` from a dict"""
        d = src_dict.copy()
        account_id = d.pop("account_id")

        created_on = isoparse(d.pop("createdOn"))

        description = d.pop("description")

        id = d.pop("id")

        name = d.pop("name")

        version_ids = cast(List[str], d.pop("versionIds"))

        _metadata = d.pop("metadata", UNSET)
        metadata: Union[Unset, CollectionMetadata]
        if isinstance(_metadata, Unset):
            metadata = UNSET
        else:
            metadata = CollectionMetadata.from_dict(_metadata)

        collection = cls(
            account_id=account_id,
            created_on=created_on,
            description=description,
            id=id,
            name=name,
            version_ids=version_ids,
            metadata=metadata,
        )

        return collection



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define


T = TypeVar("T", bound="CollectionStatsItemStatsAdditionalProperty")


@_attrs_define
class CollectionStatsItemStatsAdditionalProperty:
    """CollectionStatsItemStatsAdditionalProperty model

    Attributes:
        num_items (int):
        size_bytes (int):
    """

    num_items: int
    size_bytes: int

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        num_items = self.num_items
        size_bytes = self.size_bytes

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "numItems": num_items,
                "sizeBytes": size_bytes,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`CollectionStatsItemStatsAdditionalProperty` from a dict"""
        d = src_dict.copy()
        num_items = d.pop("numItems")

        size_bytes = d.pop("sizeBytes")

        collection_stats_item_stats_additional_property = cls(
            num_items=num_items,
            size_bytes=size_bytes,
        )

        return collection_stats_item_stats_additional_property



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define


T = TypeVar("T", bound="DataElementReference")


@_attrs_define
class DataElementReference:
    """URL-based reference to a Pigeon Data Element

    Attributes:
        ref (str):
    """

    ref: str

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        ref = self.ref

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "$ref": ref,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`DataElementReference` from a dict"""
        d = src_dict.copy()
        ref = d.pop("$ref")

        data_element_reference = cls(
            ref=ref,
        )

        return data_element_reference



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.system_configuration_workers_backend_type import (
    SystemConfigurationWorkersBackendType,
)
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="SystemConfigurationWorkers")


@_attrs_define
class SystemConfigurationWorkers:
    """SystemConfigurationWorkers model

    Attributes:
        backend_connection (Union[Unset, str]):
        backend_cycle_time (Union[Unset, float]):  Default: 15.0.
        backend_type (Union[Unset, SystemConfigurationWorkersBackendType]):
        num_worker_threads (Union[Unset, int]):  Default: 2.
    """

    backend_connection: Union[Unset, str] = UNSET
    backend_cycle_time: Union[Unset, float] = 15.0
    backend_type: Union[Unset, SystemConfigurationWorkersBackendType] = UNSET
    num_worker_threads: Union[Unset, int] = 2
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        backend_connection = self.backend_connection
        backend_cycle_time = self.backend_cycle_time
        backend_type: Union[Unset, str] = UNSET
        if not isinstance(self.backend_type, Unset):
            backend_type = self.backend_type.value

        num_worker_threads = self.num_worker_threads

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if backend_connection is not UNSET:
            field_dict["backend_connection"] = backend_connection
        if backend_cycle_time is not UNSET:
            field_dict["backend_cycle_time"] = backend_cycle_time
        if backend_type is not UNSET:
            field_dict["backend_type"] = backend_type
        if num_worker_threads is not UNSET:
            field_dict["num_worker_threads"] = num_worker_threads

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`SystemConfigurationWorkers` from a dict"""
        d = src_dict.copy()
        backend_connection = d.pop("backend_connection", UNSET)

        backend_cycle_time = d.pop("backend_cycle_time", UNSET)

        _backend_type = d.pop("backend_type", UNSET)
        backend_type: Union[Unset, SystemConfigurationWorkersBackendType]
        if isinstance(_backend_type, Unset):
            backend_type = UNSET
        else:
            backend_type = SystemConfigurationWorkersBackendType(_backend_type)

        num_worker_threads = d.pop("num_worker_threads", UNSET)

        system_configuration_workers = cls(
            backend_connection=backend_connection,
            backend_cycle_time=backend_cycle_time,
            backend_type=backend_type,
            num_worker_threads=num_worker_threads,
        )

        system_configuration_workers.additional_properties = d
        return system_configuration_workers

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any
from typing import cast
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define


T = TypeVar("T", bound="Pagination")


@_attrs_define
class Pagination:
    """Pagination model

    Attributes:
        items_per_page (int):
        next_page_url (Union[None, str]):
        total_items (int):
    """

    items_per_page: int
    next_page_url: Union[None, str]
    total_items: int

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        items_per_page = self.items_per_page
        next_page_url: Union[None, str]
        next_page_url = self.next_page_url
        total_items = self.total_items

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "items_per_page": items_per_page,
                "next_page_url": next_page_url,
                "total_items": total_items,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`Pagination` from a dict"""
        d = src_dict.copy()
        items_per_page = d.pop("items_per_page")

        def _parse_next_page_url(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        next_page_url = _parse_next_page_url(d.pop("next_page_url"))

        total_items = d.pop("total_items")

        pagination = cls(
            items_per_page=items_per_page,
            next_page_url=next_page_url,
            total_items=total_items,
        )

        return pagination



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define


T = TypeVar("T", bound="AuthProviderSigninProviderResponse200")


@_attrs_define
class AuthProviderSigninProviderResponse200:
    """AuthProviderSigninProviderResponse200 model

    Attributes:
        url (str):
    """

    url: str

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        url = self.url

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "url": url,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`AuthProviderSigninProviderResponse200` from a dict"""
        d = src_dict.copy()
        url = d.pop("url")

        auth_provider_signin_provider_response_200 = cls(
            url=url,
        )

        return auth_provider_signin_provider_response_200



from typing import Any
from typing import cast
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.item_column_type import ItemColumnType
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="ItemColumn")


@_attrs_define
class ItemColumn:
    """ItemColumn model

    Attributes:
        name (str):
        data_element_ref (Union[Unset, str]): A URL referencing a specific data element (and optional
            version) within a PDD, CDE set, or other data element
            source.
        represents_pv (Union[Unset, int, str]): When provided, this indicates that this column
            represents one of the permissible values from the
            referenced data element. The desired permissible value
            must be placed in this field, and the column type must
            be either 'Categorical' or 'Boolean'.
        type (Union[Unset, ItemColumnType]):
    """

    name: str
    data_element_ref: Union[Unset, str] = UNSET
    represents_pv: Union[Unset, int, str] = UNSET
    type: Union[Unset, ItemColumnType] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        name = self.name
        data_element_ref = self.data_element_ref
        represents_pv: Union[Unset, int, str]
        if isinstance(self.represents_pv, Unset):
            represents_pv = UNSET
        else:
            represents_pv = self.represents_pv
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "name": name,
            }
        )
        if data_element_ref is not UNSET:
            field_dict["data_element_ref"] = data_element_ref
        if represents_pv is not UNSET:
            field_dict["represents_pv"] = represents_pv
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`ItemColumn` from a dict"""
        d = src_dict.copy()
        name = d.pop("name")

        data_element_ref = d.pop("data_element_ref", UNSET)

        def _parse_represents_pv(data: object) -> Union[Unset, int, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Unset, int, str], data)

        represents_pv = _parse_represents_pv(d.pop("represents_pv", UNSET))

        _type = d.pop("type", UNSET)
        type: Union[Unset, ItemColumnType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = ItemColumnType(_type)

        item_column = cls(
            name=name,
            data_element_ref=data_element_ref,
            represents_pv=represents_pv,
            type=type,
        )

        return item_column



from enum import Enum


class CollectionsDictionariesCreateNewDataElementVersionDataElementDataType(str, Enum):
    BOOLEAN = "boolean"
    NUMBER = "number"
    STRING = "string"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define

from ..models.search_search_dictionaries_body_options import (
    SearchSearchDictionariesBodyOptions,
)


T = TypeVar("T", bound="SearchSearchDictionariesBody")


@_attrs_define
class SearchSearchDictionariesBody:
    """SearchSearchDictionariesBody model

    Attributes:
        options (SearchSearchDictionariesBodyOptions):
        query (str):
        source (str):
    """

    options: "SearchSearchDictionariesBodyOptions"
    query: str
    source: str

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        options = self.options.to_dict()
        query = self.query
        source = self.source

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "options": options,
                "query": query,
                "source": source,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`SearchSearchDictionariesBody` from a dict"""
        d = src_dict.copy()
        options = SearchSearchDictionariesBodyOptions.from_dict(d.pop("options"))

        query = d.pop("query")

        source = d.pop("source")

        search_search_dictionaries_body = cls(
            options=options,
            query=query,
            source=source,
        )

        return search_search_dictionaries_body



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define

from ..models.data_element_reference import DataElementReference
from ..models.pagination import Pagination


T = TypeVar("T", bound="CollectionsDictionariesListDataElementRefsResponse200")


@_attrs_define
class CollectionsDictionariesListDataElementRefsResponse200:
    """CollectionsDictionariesListDataElementRefsResponse200 model

    Attributes:
        elements (List['DataElementReference']):
        pagination (Pagination):
    """

    elements: List["DataElementReference"]
    pagination: "Pagination"

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        elements = []
        for elements_item_data in self.elements:
            elements_item = elements_item_data.to_dict()
            elements.append(elements_item)

        pagination = self.pagination.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "elements": elements,
                "pagination": pagination,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`CollectionsDictionariesListDataElementRefsResponse200` from a dict"""
        d = src_dict.copy()
        elements = []
        _elements = d.pop("elements")
        for elements_item_data in _elements:
            elements_item = DataElementReference.from_dict(elements_item_data)

            elements.append(elements_item)

        pagination = Pagination.from_dict(d.pop("pagination"))

        collections_dictionaries_list_data_element_refs_response_200 = cls(
            elements=elements,
            pagination=pagination,
        )

        return collections_dictionaries_list_data_element_refs_response_200



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field


T = TypeVar("T", bound="ItemParserOptions")


@_attrs_define
class ItemParserOptions:
    """ItemParserOptions model"""

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`ItemParserOptions` from a dict"""
        d = src_dict.copy()
        item_parser_options = cls()

        item_parser_options.additional_properties = d
        return item_parser_options

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import argparse
import json
import textwrap
import types
from typing import Any
from typing import Callable

import yaml
from pypigeon.client import PigeonClient


class BaseCommands:
    _func_args: dict[Callable[..., None], list[tuple[tuple[Any, ...], dict[str, Any]]]]

    def __init__(self, client: PigeonClient, args: argparse.Namespace) -> None:
        self.client = client
        self.core = client._client
        self.args = args

    def _output(self, data: Any) -> None:
        if self.args.output_type == "yaml":
            print(yaml.dump(data))

        elif self.args.output_type == "json":
            print(json.dumps(data, indent=2))

        else:
            raise ValueError(f"unknown output type: {self.args.output_type}")

    @classmethod
    def _add_to_parser(
        cls, subparsers: "argparse._SubParsersAction[argparse.ArgumentParser]"
    ) -> None:
        for method in dir(cls):
            if method.startswith("_"):
                continue

            action = getattr(cls, method)
            action_doc = action.__doc__ if action.__doc__ is not None else ""
            help_line = (
                action_doc.splitlines()[0].strip().rstrip(".") if action_doc else ""
            )
            parser = subparsers.add_parser(
                method.replace("_", "-"),
                help=help_line,
                description=(
                    help_line
                    + "\n\n"
                    + textwrap.dedent("\n".join(action_doc.split("\n")[1:]))
                ),
                formatter_class=argparse.RawDescriptionHelpFormatter,
            )

            if type(action) == type(object) and issubclass(action, BaseCommands):
                action._add_to_parser(parser.add_subparsers(title="subcommands"))

            elif isinstance(action, types.FunctionType):
                if hasattr(cls, "_func_args"):
                    for args, kwargs in cls._func_args.get(action, []):
                        parser.add_argument(*args, **kwargs)
                parser.set_defaults(cls=cls, func=action)

    @classmethod
    def _with_arg(cls, *args: Any, **kwargs: Any) -> Callable[..., Callable[..., None]]:
        def _inner(func: Callable[..., None]) -> Callable[..., None]:
            if not hasattr(cls, "_func_args"):
                cls._func_args = {}
            cls._func_args.setdefault(func, []).insert(0, (args, kwargs))
            return func

        return _inner



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.collections_dictionaries_create_a_new_data_element_data_element_data_type import (
    CollectionsDictionariesCreateANewDataElementDataElementDataType,
)
from ..models.data_element_concept import DataElementConcept
from ..models.data_element_definition import DataElementDefinition
from ..models.data_element_permissible_values_external_reference import (
    DataElementPermissibleValuesExternalReference,
)
from ..models.data_element_permissible_values_number_range import (
    DataElementPermissibleValuesNumberRange,
)
from ..models.data_element_permissible_values_text_range import (
    DataElementPermissibleValuesTextRange,
)
from ..models.data_element_permissible_values_value_set import (
    DataElementPermissibleValuesValueSet,
)
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="CollectionsDictionariesCreateANewDataElementDataElement")


@_attrs_define
class CollectionsDictionariesCreateANewDataElementDataElement:
    """Pigeon Data Element

    Attributes:
        data_type (CollectionsDictionariesCreateANewDataElementDataElementDataType): The data type valid for this data
            element.
        name (str): The title or name of the data element.
        concepts (Union[Unset, List['DataElementConcept']]): Linked concepts for the data element.
        definition (Union[Unset, List['DataElementDefinition']]): Any definitions that describe or inform the context of
            the data element.
        permissible_values (Union[Unset, List[Union['DataElementPermissibleValuesExternalReference',
            'DataElementPermissibleValuesNumberRange', 'DataElementPermissibleValuesTextRange',
            'DataElementPermissibleValuesValueSet']]]): Describes the values that this data element properly represents.
        permit_null (Union[Unset, bool]): If true, nulls will not cause validation failures. Default: False.
        sensitivity (Union[Unset, bool]): Whether this data element references sensitive data (PHI, PII, etc.)
    """

    data_type: CollectionsDictionariesCreateANewDataElementDataElementDataType
    name: str
    concepts: Union[Unset, List["DataElementConcept"]] = UNSET
    definition: Union[Unset, List["DataElementDefinition"]] = UNSET
    permissible_values: Union[
        Unset,
        List[
            Union[
                "DataElementPermissibleValuesExternalReference",
                "DataElementPermissibleValuesNumberRange",
                "DataElementPermissibleValuesTextRange",
                "DataElementPermissibleValuesValueSet",
            ]
        ],
    ] = UNSET
    permit_null: Union[Unset, bool] = False
    sensitivity: Union[Unset, bool] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        data_type = self.data_type.value
        name = self.name
        concepts: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.concepts, Unset):
            concepts = []
            for concepts_item_data in self.concepts:
                concepts_item = concepts_item_data.to_dict()
                concepts.append(concepts_item)

        definition: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.definition, Unset):
            definition = []
            for definition_item_data in self.definition:
                definition_item = definition_item_data.to_dict()
                definition.append(definition_item)

        permissible_values: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.permissible_values, Unset):
            permissible_values = []
            for permissible_values_item_data in self.permissible_values:
                permissible_values_item: Dict[str, Any]
                if isinstance(
                    permissible_values_item_data,
                    DataElementPermissibleValuesNumberRange,
                ):
                    permissible_values_item = permissible_values_item_data.to_dict()
                elif isinstance(
                    permissible_values_item_data, DataElementPermissibleValuesTextRange
                ):
                    permissible_values_item = permissible_values_item_data.to_dict()
                elif isinstance(
                    permissible_values_item_data, DataElementPermissibleValuesValueSet
                ):
                    permissible_values_item = permissible_values_item_data.to_dict()
                else:
                    permissible_values_item = permissible_values_item_data.to_dict()

                permissible_values.append(permissible_values_item)

        permit_null = self.permit_null
        sensitivity = self.sensitivity

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "dataType": data_type,
                "name": name,
            }
        )
        if concepts is not UNSET:
            field_dict["concepts"] = concepts
        if definition is not UNSET:
            field_dict["definition"] = definition
        if permissible_values is not UNSET:
            field_dict["permissibleValues"] = permissible_values
        if permit_null is not UNSET:
            field_dict["permitNull"] = permit_null
        if sensitivity is not UNSET:
            field_dict["sensitivity"] = sensitivity

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`CollectionsDictionariesCreateANewDataElementDataElement` from a dict"""
        d = src_dict.copy()
        data_type = CollectionsDictionariesCreateANewDataElementDataElementDataType(
            d.pop("dataType")
        )

        name = d.pop("name")

        concepts = []
        _concepts = d.pop("concepts", UNSET)
        for concepts_item_data in _concepts or []:
            concepts_item = DataElementConcept.from_dict(concepts_item_data)

            concepts.append(concepts_item)

        definition = []
        _definition = d.pop("definition", UNSET)
        for definition_item_data in _definition or []:
            definition_item = DataElementDefinition.from_dict(definition_item_data)

            definition.append(definition_item)

        permissible_values = []
        _permissible_values = d.pop("permissibleValues", UNSET)
        for permissible_values_item_data in _permissible_values or []:

            def _parse_permissible_values_item(
                data: object,
            ) -> Union[
                "DataElementPermissibleValuesExternalReference",
                "DataElementPermissibleValuesNumberRange",
                "DataElementPermissibleValuesTextRange",
                "DataElementPermissibleValuesValueSet",
            ]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    componentsschemasdata_element_permissible_values_type_0 = (
                        DataElementPermissibleValuesNumberRange.from_dict(data)
                    )

                    return componentsschemasdata_element_permissible_values_type_0
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    componentsschemasdata_element_permissible_values_type_1 = (
                        DataElementPermissibleValuesTextRange.from_dict(data)
                    )

                    return componentsschemasdata_element_permissible_values_type_1
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    componentsschemasdata_element_permissible_values_type_2 = (
                        DataElementPermissibleValuesValueSet.from_dict(data)
                    )

                    return componentsschemasdata_element_permissible_values_type_2
                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemasdata_element_permissible_values_type_3 = (
                    DataElementPermissibleValuesExternalReference.from_dict(data)
                )

                return componentsschemasdata_element_permissible_values_type_3

            permissible_values_item = _parse_permissible_values_item(
                permissible_values_item_data
            )

            permissible_values.append(permissible_values_item)

        permit_null = d.pop("permitNull", UNSET)

        sensitivity = d.pop("sensitivity", UNSET)

        collections_dictionaries_create_a_new_data_element_data_element = cls(
            data_type=data_type,
            name=name,
            concepts=concepts,
            definition=definition,
            permissible_values=permissible_values,
            permit_null=permit_null,
            sensitivity=sensitivity,
        )

        return collections_dictionaries_create_a_new_data_element_data_element



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define

from ..models.item_column_type import ItemColumnType
from ..models.search_dictionaries_by_item_column_hits_search_dictionaries_hit import (
    SearchDictionariesByItemColumnHitsSearchDictionariesHit,
)


T = TypeVar("T", bound="SearchDictionariesByItemColumnHits")


@_attrs_define
class SearchDictionariesByItemColumnHits:
    """SearchDictionariesByItemColumnHits model

    Attributes:
        hits (List['SearchDictionariesByItemColumnHitsSearchDictionariesHit']):
        name (str):
        type_as_parsed (ItemColumnType):
    """

    hits: List["SearchDictionariesByItemColumnHitsSearchDictionariesHit"]
    name: str
    type_as_parsed: ItemColumnType

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        hits = []
        for hits_item_data in self.hits:
            hits_item = hits_item_data.to_dict()
            hits.append(hits_item)

        name = self.name
        type_as_parsed = self.type_as_parsed.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "hits": hits,
                "name": name,
                "type_as_parsed": type_as_parsed,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`SearchDictionariesByItemColumnHits` from a dict"""
        d = src_dict.copy()
        hits = []
        _hits = d.pop("hits")
        for hits_item_data in _hits:
            hits_item = (
                SearchDictionariesByItemColumnHitsSearchDictionariesHit.from_dict(
                    hits_item_data
                )
            )

            hits.append(hits_item)

        name = d.pop("name")

        type_as_parsed = ItemColumnType(d.pop("type_as_parsed"))

        search_dictionaries_by_item_column_hits = cls(
            hits=hits,
            name=name,
            type_as_parsed=type_as_parsed,
        )

        return search_dictionaries_by_item_column_hits



from pypigeon import item_io

from .base_commands import BaseCommands


class PddCommands(BaseCommands):
    """Operations on PDDs (data dictionaries)"""

    @BaseCommands._with_arg("collection_id")
    @BaseCommands._with_arg("filename")
    @BaseCommands._with_arg("--item-name")
    @BaseCommands._with_arg("--folder-id", default="ROOT")
    def upload(self) -> None:
        """Upload a PDD from a local file."""

        item_name = self.args.item_name or self.args.filename
        writer = item_io.PigeonItemCreatorText(
            self.args.collection_id,
            "LIVE",
            item_name,
            self.core,
            folder_id=self.args.folder_id,
            type="dictionary",
        )
        with writer as ofp:
            with open(self.args.filename) as ifp:
                ofp.write(ifp.read())



# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html
# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

project = "PyPigeon"
copyright = "2023, BioTeam"
author = "BioTeam"
release = "0.2.2"

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.autosummary",
    "sphinx.ext.intersphinx",
    "sphinx.ext.viewcode",
    "sphinx.ext.napoleon",
    "sphinx.ext.githubpages",
]

autosummary_generate = True
autodoc_inherit_docstrings = True
autodoc_typehints = "description"

templates_path = ["_templates"]

intersphinx_mapping = {
    "python": ("https://docs.python.org/3/", None),
    "pandas": ("https://pandas.pydata.org/docs/", None),
}

# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = "alabaster"
html_theme_options = {
    "description": "an easy-to-use client library for Pigeon",
    "github_user": "bioteam",
    "github_repo": "project-pigeon",
}
html_static_path = ["_static"]



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.identity import Identity
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="SessionUser")


@_attrs_define
class SessionUser:
    """SessionUser model

    Attributes:
        identity (Identity):
        name (str):
        account_id (Union[Unset, str]):
        email (Union[Unset, str]):
        image (Union[Unset, str]):
    """

    identity: "Identity"
    name: str
    account_id: Union[Unset, str] = UNSET
    email: Union[Unset, str] = UNSET
    image: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        identity = self.identity.to_dict()
        name = self.name
        account_id = self.account_id
        email = self.email
        image = self.image

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "identity": identity,
                "name": name,
            }
        )
        if account_id is not UNSET:
            field_dict["account_id"] = account_id
        if email is not UNSET:
            field_dict["email"] = email
        if image is not UNSET:
            field_dict["image"] = image

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`SessionUser` from a dict"""
        d = src_dict.copy()
        identity = Identity.from_dict(d.pop("identity"))

        name = d.pop("name")

        account_id = d.pop("account_id", UNSET)

        email = d.pop("email", UNSET)

        image = d.pop("image", UNSET)

        session_user = cls(
            identity=identity,
            name=name,
            account_id=account_id,
            email=email,
            image=image,
        )

        return session_user



"""Activate a placeholder session ID using the current session"""
from http import HTTPStatus
from typing import Any
from typing import cast
from typing import Dict
from typing import Union

import httpx

from ... import errors
from ...client import AuthenticatedClient
from ...client import Client
from ...models.auth_activate_session_body import AuthActivateSessionBody
from ...models.server_error import ServerError
from ...types import Response


def _get_kwargs(
    *,
    body: AuthActivateSessionBody,
) -> Dict[str, Any]:
    headers: Dict[str, Any] = {}

    _kwargs: Dict[str, Any] = {
        "method": "post",
        "url": "/auth/session/activate",
    }

    _body = body.to_dict()

    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Union[Any, ServerError]:
    if response.status_code == HTTPStatus.NO_CONTENT:
        response_204 = cast(Any, {"attribute": "None", "return_type": "None"})
        return response_204
    if response.status_code == HTTPStatus.BAD_REQUEST:
        response_400 = ServerError.from_dict(response.json())

        return response_400
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        response_500 = ServerError.from_dict(response.json())

        return response_500

    raise errors.UnexpectedStatus(response.status_code, response.content)


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Any, ServerError]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    body: AuthActivateSessionBody,
) -> Response[Union[Any, ServerError]]:
    """Activate a placeholder session ID using the current session

    By providing an activation code, the related placeholder
    session will assume the identity of the current session.

    This method cannot be used to reactivate expired sessions, nor
    can it be used by an assumed identity.

    Args:
        body (AuthActivateSessionBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, ServerError]]
    """

    kwargs = _get_kwargs(
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: Union[AuthenticatedClient, Client],
    body: AuthActivateSessionBody,
) -> Union[Any]:
    """Activate a placeholder session ID using the current session

    By providing an activation code, the related placeholder
    session will assume the identity of the current session.

    This method cannot be used to reactivate expired sessions, nor
    can it be used by an assumed identity.

    Args:
        body (AuthActivateSessionBody):

    Raises:
        errors.UnexpectedStatus: If the server returns a status code greater than or equal to 300.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any]
    """

    response = sync_detailed(
        client=client,
        body=body,
    )
    if isinstance(response.parsed, ServerError):
        raise errors.UnexpectedStatus(response.status_code, response.content)

    return response.parsed


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    body: AuthActivateSessionBody,
) -> Response[Union[Any, ServerError]]:
    """Activate a placeholder session ID using the current session

    By providing an activation code, the related placeholder
    session will assume the identity of the current session.

    This method cannot be used to reactivate expired sessions, nor
    can it be used by an assumed identity.

    Args:
        body (AuthActivateSessionBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, ServerError]]
    """

    kwargs = _get_kwargs(
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],
    body: AuthActivateSessionBody,
) -> Union[Any]:
    """Activate a placeholder session ID using the current session

    By providing an activation code, the related placeholder
    session will assume the identity of the current session.

    This method cannot be used to reactivate expired sessions, nor
    can it be used by an assumed identity.

    Args:
        body (AuthActivateSessionBody):

    Raises:
        errors.UnexpectedStatus: If the server returns a status code greater than or equal to 300.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any]
    """

    response = await asyncio_detailed(
        client=client,
        body=body,
    )
    if isinstance(response.parsed, ServerError):
        raise errors.UnexpectedStatus(response.status_code, response.content)

    return response.parsed



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="TableError")


@_attrs_define
class TableError:
    """TableError model

    Attributes:
        detail (str):
        title (str):
        source (Union[Unset, str]):
    """

    detail: str
    title: str
    source: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        detail = self.detail
        title = self.title
        source = self.source

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "detail": detail,
                "title": title,
            }
        )
        if source is not UNSET:
            field_dict["source"] = source

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`TableError` from a dict"""
        d = src_dict.copy()
        detail = d.pop("detail")

        title = d.pop("title")

        source = d.pop("source", UNSET)

        table_error = cls(
            detail=detail,
            title=title,
            source=source,
        )

        return table_error



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define

from ..models.session_user import SessionUser


T = TypeVar("T", bound="AuthGetSessionResponse200")


@_attrs_define
class AuthGetSessionResponse200:
    """AuthGetSessionResponse200 model

    Attributes:
        expires (str):
        user (SessionUser):
    """

    expires: str
    user: "SessionUser"

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        expires = self.expires
        user = self.user.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "expires": expires,
                "user": user,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`AuthGetSessionResponse200` from a dict"""
        d = src_dict.copy()
        expires = d.pop("expires")

        user = SessionUser.from_dict(d.pop("user"))

        auth_get_session_response_200 = cls(
            expires=expires,
            user=user,
        )

        return auth_get_session_response_200



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="DataElementPermissibleValuesExternalReferenceExternalReference")


@_attrs_define
class DataElementPermissibleValuesExternalReferenceExternalReference:
    """A reference to an external source of permissible values.

    Attributes:
        reference_id (str):
        source (str):
        url (Union[Unset, str]):
    """

    reference_id: str
    source: str
    url: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        reference_id = self.reference_id
        source = self.source
        url = self.url

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "referenceId": reference_id,
                "source": source,
            }
        )
        if url is not UNSET:
            field_dict["url"] = url

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`DataElementPermissibleValuesExternalReferenceExternalReference` from a dict"""
        d = src_dict.copy()
        reference_id = d.pop("referenceId")

        source = d.pop("source")

        url = d.pop("url", UNSET)

        data_element_permissible_values_external_reference_external_reference = cls(
            reference_id=reference_id,
            source=source,
            url=url,
        )

        return data_element_permissible_values_external_reference_external_reference



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="AuthAuthenticateUserAuthenticationRequest")


@_attrs_define
class AuthAuthenticateUserAuthenticationRequest:
    """AuthAuthenticateUserAuthenticationRequest model

    Attributes:
        callback_url (str):
        csrf_token (str):
        password (str):
        user_name (str):
        json (Union[Unset, bool]):  Default: True.
        redirect (Union[Unset, bool]):  Default: True.
    """

    callback_url: str
    csrf_token: str
    password: str
    user_name: str
    json: Union[Unset, bool] = True
    redirect: Union[Unset, bool] = True

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        callback_url = self.callback_url
        csrf_token = self.csrf_token
        password = self.password
        user_name = self.user_name
        json = self.json
        redirect = self.redirect

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "callbackUrl": callback_url,
                "csrfToken": csrf_token,
                "password": password,
                "user_name": user_name,
            }
        )
        if json is not UNSET:
            field_dict["json"] = json
        if redirect is not UNSET:
            field_dict["redirect"] = redirect

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`AuthAuthenticateUserAuthenticationRequest` from a dict"""
        d = src_dict.copy()
        callback_url = d.pop("callbackUrl")

        csrf_token = d.pop("csrfToken")

        password = d.pop("password")

        user_name = d.pop("user_name")

        json = d.pop("json", UNSET)

        redirect = d.pop("redirect", UNSET)

        auth_authenticate_user_authentication_request = cls(
            callback_url=callback_url,
            csrf_token=csrf_token,
            password=password,
            user_name=user_name,
            json=json,
            redirect=redirect,
        )

        return auth_authenticate_user_authentication_request



from typing import Any
from typing import cast
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.new_collection_version_metadata import NewCollectionVersionMetadata
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="NewCollectionVersion")


@_attrs_define
class NewCollectionVersion:
    """Collection version details

    Attributes:
        description (str): The description of the version
        item_ids (Union[Unset, List[str]]): The items to include in this collection version
        metadata (Union[Unset, NewCollectionVersionMetadata]): Arbitrary user-defined metadata, key-value pairs
    """

    description: str
    item_ids: Union[Unset, List[str]] = UNSET
    metadata: Union[Unset, "NewCollectionVersionMetadata"] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        description = self.description
        item_ids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.item_ids, Unset):
            item_ids = self.item_ids

        metadata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = self.metadata.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "description": description,
            }
        )
        if item_ids is not UNSET:
            field_dict["itemIds"] = item_ids
        if metadata is not UNSET:
            field_dict["metadata"] = metadata

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`NewCollectionVersion` from a dict"""
        d = src_dict.copy()
        description = d.pop("description")

        item_ids = cast(List[str], d.pop("itemIds", UNSET))

        _metadata = d.pop("metadata", UNSET)
        metadata: Union[Unset, NewCollectionVersionMetadata]
        if isinstance(_metadata, Unset):
            metadata = UNSET
        else:
            metadata = NewCollectionVersionMetadata.from_dict(_metadata)

        new_collection_version = cls(
            description=description,
            item_ids=item_ids,
            metadata=metadata,
        )

        return new_collection_version



"""Search a data dictionary for data elements compatible with an item's columns
"""
from http import HTTPStatus
from typing import Any
from typing import Dict
from typing import Union

import httpx

from ... import errors
from ...client import AuthenticatedClient
from ...client import Client
from ...models.search_dictionaries_by_item_response import (
    SearchDictionariesByItemResponse,
)
from ...models.search_search_dictionaries_by_item_body import (
    SearchSearchDictionariesByItemBody,
)
from ...models.server_error import ServerError
from ...types import Response


def _get_kwargs(
    *,
    body: SearchSearchDictionariesByItemBody,
) -> Dict[str, Any]:
    headers: Dict[str, Any] = {}

    _kwargs: Dict[str, Any] = {
        "method": "post",
        "url": "/search/dictionaries/byitem",
    }

    _body = body.to_dict()

    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Union[SearchDictionariesByItemResponse, ServerError]:
    if response.status_code == HTTPStatus.OK:
        response_200 = SearchDictionariesByItemResponse.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.BAD_REQUEST:
        response_400 = ServerError.from_dict(response.json())

        return response_400
    if response.status_code == HTTPStatus.UNSUPPORTED_MEDIA_TYPE:
        response_415 = ServerError.from_dict(response.json())

        return response_415
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        response_500 = ServerError.from_dict(response.json())

        return response_500

    raise errors.UnexpectedStatus(response.status_code, response.content)


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[SearchDictionariesByItemResponse, ServerError]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    body: SearchSearchDictionariesByItemBody,
) -> Response[Union[SearchDictionariesByItemResponse, ServerError]]:
    """Search a data dictionary for data elements compatible with an item's columns

    This performs a 'forward search' for data elements from the
    provided source that are likely to describe columns in the
    provided item. The results are returned on a per-column basis,
    paginated across the columns in the provided item.

    The current method uses column names as full-text search
    queries against the text extracted from the data element
    definition, including its title, descriptions, and concept
    codes.

    Args:
        body (SearchSearchDictionariesByItemBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[SearchDictionariesByItemResponse, ServerError]]
    """

    kwargs = _get_kwargs(
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    body: SearchSearchDictionariesByItemBody,
) -> Union[SearchDictionariesByItemResponse]:
    """Search a data dictionary for data elements compatible with an item's columns

    This performs a 'forward search' for data elements from the
    provided source that are likely to describe columns in the
    provided item. The results are returned on a per-column basis,
    paginated across the columns in the provided item.

    The current method uses column names as full-text search
    queries against the text extracted from the data element
    definition, including its title, descriptions, and concept
    codes.

    Args:
        body (SearchSearchDictionariesByItemBody):

    Raises:
        errors.UnexpectedStatus: If the server returns a status code greater than or equal to 300.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[SearchDictionariesByItemResponse]
    """

    response = sync_detailed(
        client=client,
        body=body,
    )
    if isinstance(response.parsed, ServerError):
        raise errors.UnexpectedStatus(response.status_code, response.content)

    return response.parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    body: SearchSearchDictionariesByItemBody,
) -> Response[Union[SearchDictionariesByItemResponse, ServerError]]:
    """Search a data dictionary for data elements compatible with an item's columns

    This performs a 'forward search' for data elements from the
    provided source that are likely to describe columns in the
    provided item. The results are returned on a per-column basis,
    paginated across the columns in the provided item.

    The current method uses column names as full-text search
    queries against the text extracted from the data element
    definition, including its title, descriptions, and concept
    codes.

    Args:
        body (SearchSearchDictionariesByItemBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[SearchDictionariesByItemResponse, ServerError]]
    """

    kwargs = _get_kwargs(
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    body: SearchSearchDictionariesByItemBody,
) -> Union[SearchDictionariesByItemResponse]:
    """Search a data dictionary for data elements compatible with an item's columns

    This performs a 'forward search' for data elements from the
    provided source that are likely to describe columns in the
    provided item. The results are returned on a per-column basis,
    paginated across the columns in the provided item.

    The current method uses column names as full-text search
    queries against the text extracted from the data element
    definition, including its title, descriptions, and concept
    codes.

    Args:
        body (SearchSearchDictionariesByItemBody):

    Raises:
        errors.UnexpectedStatus: If the server returns a status code greater than or equal to 300.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[SearchDictionariesByItemResponse]
    """

    response = await asyncio_detailed(
        client=client,
        body=body,
    )
    if isinstance(response.parsed, ServerError):
        raise errors.UnexpectedStatus(response.status_code, response.content)

    return response.parsed



from pypigeon.pigeon_core import Client
from pypigeon.pigeon_core import Login
from pypigeon.pigeon_core.api.auth import auth_get_session
from pypigeon.pigeon_core.api.user import user_get_user_apc_outbox


def test_login_password(httpx_mock):
    httpx_mock.add_response(
        url="https://pigeon.test/api/v1/auth/providers",
        json={
            "credentials": {
                "id": "credentials",
                "name": "Credentials",
                "type": "credentials",
                "signinUrl": "https://pigeon.test/api/v1/auth/signin",
                "callbackUrl": "https://pigeon.test/api/v1/auth/callback/credentials",
            }
        },
    )
    httpx_mock.add_response(
        url="https://pigeon.test/api/v1/auth/csrf",
        json={"csrfToken": "a1b2c3d4"},
    )
    httpx_mock.add_response(
        url="https://pigeon.test/api/v1/auth/callback/credentials",
        method="POST",
        match_json={
            "user_name": "testuser",
            "password": "testpassword",
            "callbackUrl": "http://null",
            "csrfToken": "a1b2c3d4",
            "json": True,
            "redirect": True,
        },
        json={"url": "http://null"},
        headers={"Set-Cookie": "session=something"},
    )

    client = Client(base_url="https://pigeon.test/api/v1")
    client = Login(client).with_password("testuser", "testpassword")
    assert client.token == "something"

    httpx_mock.add_response(
        url="https://pigeon.test/api/v1/user/testuser/outbox",
        match_headers={"Authorization": "Bearer something"},
        json={
            "@context": "https://www.w3.org/ns/activitystreams",
            "id": "https://pigeon.test/api/v1/user/testuser/outbox",
            "type": "OrderedCollectionPage",
            "totalItems": 0,
            "orderedItems": [],
        },
    )

    response = user_get_user_apc_outbox.sync(user_name="testuser", client=client)
    assert response.type == "OrderedCollectionPage"


def test_login_token(httpx_mock):
    httpx_mock.add_response(
        url="https://pigeon.test/api/v1/auth/session",
        match_headers={"Authorization": "Bearer something"},
        json={
            "expires": "2024-01-01T00:00:00+00:00",
            "user": {
                "identity": {
                    "primary_id": "aaaabbbb",
                    "fqda": "someone@localhost",
                    "user_name": "someone",
                },
                "name": "Someone",
                "email": None,
                "image": None,
                "account_id": "xxxxyyyy",
            },
        },
    )

    client = Client(base_url="https://pigeon.test/api/v1")
    client = Login(client).with_session_id("something")
    assert client.token == "something"

    response = auth_get_session.sync(client=client)
    assert response.user.name == "Someone"



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define

from ..models.termset import Termset


T = TypeVar("T", bound="SearchGetCollectionTermsResponse200")


@_attrs_define
class SearchGetCollectionTermsResponse200:
    """SearchGetCollectionTermsResponse200 model

    Attributes:
        terms (Termset):
    """

    terms: "Termset"

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        terms = self.terms.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "terms": terms,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`SearchGetCollectionTermsResponse200` from a dict"""
        d = src_dict.copy()
        terms = Termset.from_dict(d.pop("terms"))

        search_get_collection_terms_response_200 = cls(
            terms=terms,
        )

        return search_get_collection_terms_response_200



import datetime
from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define
from dateutil.parser import isoparse

from ..models.group_role import GroupRole
from ..models.user_membership import UserMembership
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="Group")


@_attrs_define
class Group:
    """Group model

    Attributes:
        created_on (datetime.datetime):
        group_name (str):
        subject_id (str):
        id (Union[Unset, str]):
        role (Union[Unset, GroupRole]):
        users (Union[Unset, List['UserMembership']]):
    """

    created_on: datetime.datetime
    group_name: str
    subject_id: str
    id: Union[Unset, str] = UNSET
    role: Union[Unset, GroupRole] = UNSET
    users: Union[Unset, List["UserMembership"]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        created_on = self.created_on.isoformat()
        group_name = self.group_name
        subject_id = self.subject_id
        id = self.id
        role: Union[Unset, str] = UNSET
        if not isinstance(self.role, Unset):
            role = self.role.value

        users: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.users, Unset):
            users = []
            for users_item_data in self.users:
                users_item = users_item_data.to_dict()
                users.append(users_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "created_on": created_on,
                "group_name": group_name,
                "subject_id": subject_id,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if role is not UNSET:
            field_dict["role"] = role
        if users is not UNSET:
            field_dict["users"] = users

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`Group` from a dict"""
        d = src_dict.copy()
        created_on = isoparse(d.pop("created_on"))

        group_name = d.pop("group_name")

        subject_id = d.pop("subject_id")

        id = d.pop("id", UNSET)

        _role = d.pop("role", UNSET)
        role: Union[Unset, GroupRole]
        if isinstance(_role, Unset):
            role = UNSET
        else:
            role = GroupRole(_role)

        users = []
        _users = d.pop("users", UNSET)
        for users_item_data in _users or []:
            users_item = UserMembership.from_dict(users_item_data)

            users.append(users_item)

        group = cls(
            created_on=created_on,
            group_name=group_name,
            subject_id=subject_id,
            id=id,
            role=role,
            users=users,
        )

        return group
